@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.3.8  Copyright by Tobi Oetiker, 1997-2009
+ * RRDtool 1.3.9  Copyright by Tobi Oetiker, 1997-2009
  *****************************************************************************
  * rrd_info  Get Information about the configuration of an RRD
  *****************************************************************************/
@@ -24,9 +24,13 @@ char     *sprintf_alloc(
     char *fmt,
     ...)
 {
-    int       maxlen = 1024 + strlen(fmt);
     char     *str = NULL;
     va_list   argp;
+#ifdef HAVE_VASPRINTF
+	va_start( argp, fmt );
+	vasprintf( &str, fmt, argp );
+#else
+    int       maxlen = 1024 + strlen(fmt);
     str = (char*)(malloc(sizeof(char) * (maxlen + 1)));
     if (str != NULL) {
         va_start(argp, fmt);
@@ -36,6 +40,7 @@ char     *sprintf_alloc(
         vsprintf(str, fmt, argp);
 #endif
     }
+#endif // HAVE_VASPRINTF
     va_end(argp);
     return str;
 }
@@ -127,6 +132,11 @@ rrd_info_t *rrd_info_r(
 
     for (i = 0; i < rrd.stat_head->ds_cnt; i++) {
 
+        info.u_cnt=i;
+        cd= rrd_info_push(cd,sprintf_alloc("ds[%s].index",
+                                     rrd.ds_def[i].ds_nam),
+                     RD_I_CNT, info);
+    
         info.u_str = rrd.ds_def[i].dst;
         cd = rrd_info_push(cd, sprintf_alloc("ds[%s].type",
                                              rrd.ds_def[i].ds_nam),
